/*

*/

#include "MyWire.h"
#include <TimerTCC0.h>



#define SSV_ADDRESS 0x0a

char cmdbuf[32];
int cmdbufindex;
char debug[32];
byte  timerFlag;

void timerIsr()
{    
    timerFlag = 1;
}

byte SSV3232_GetStatus(void)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0xF7);
    Wire.endTransmission(false);
    Wire.requestFrom(SSV_ADDRESS,1);
    return Wire.read();  
}

int SSV3232_GetAmbientTemp(void)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0x03);
    Wire.endTransmission(false);
    Wire.requestFrom(SSV_ADDRESS,2);
    int dl= Wire.read();
    int dh= Wire.read();
    return((dh<<8)&0xFF00)+(dl&0xFF);
}

int SSV3232_GetEmissivity(void)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0xF2);
    Wire.endTransmission(false);
    Wire.requestFrom(SSV_ADDRESS,2);
    int dl= Wire.read();
    int dh= Wire.read();
    return((dh<<8)&0xFF00)+(dl&0xFF);
}

void SSV3232_SetEmissivity(unsigned int param)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0xF2);
    Wire.write(param&0xFF);
    Wire.write((param>>8)&0xFF);
    Wire.endTransmission(true);
}

void SSV3232_SetCtrl(byte param)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0xF7);
    Wire.write(param);
    Wire.endTransmission(true);
}


int16_t arrayBuf[1024];


void SSV3232_GetArrayData(void)
{
    Wire.beginTransmission(SSV_ADDRESS);
    Wire.write(0x02);
    Wire.endTransmission(false);

    Wire.requestFrom(SSV_ADDRESS,2048,false);

    int16_t *pdat;
    pdat=arrayBuf;
    for(int i=0;i<1024;i++){
      int16_t i1=Wire.read();
      int16_t i2=Wire.read();
      *pdat++  = ((i2<<8)&0xFF00)|(i1&0xFF);
    }
    
    Wire.endTransmission(true);

}




// the setup function runs once when you press reset or power the board
void setup() {
  //pinMode(LED_BUILTIN, OUTPUT);

  cmdbufindex=0;
  timerFlag=0;

  SerialUSB.begin(230400);//意味なし。

  Wire.begin();
  Wire.setClock(400000);

  SSV3232_SetCtrl(0x03);

  TimerTcc0.initialize(125000);
  TimerTcc0.attachInterrupt(timerIsr);

}

//SerialUSBでもこまめにprintするよりまとめる方が速くなる。
//1行分まとめて送信。
void dump(void)
{
  char s[256];
  int16_t *pdat = arrayBuf;
  for(int y=0;y<32;y++){
    char*p =s;
    for(int x=0;x<32;x++){
      int n=sprintf(p,"%04X,",*pdat++);
      p+=n;
    }
    p--;
    *p=0; //行末のカンマを0に置き換え
    SerialUSB.println(s);
  }
}


void proccmd(void)
{
  
  unsigned int cmd,param;
  if(!SerialUSB) return;
  while(SerialUSB.available()){
    byte c = SerialUSB.read();
    if(c=='W')cmdbufindex=0;
    else if(c=='\r'){
      cmdbuf[cmdbufindex]=0;
      
      if(cmdbufindex>=3){
        cmdbuf[3]=0;
        cmd=strtol(cmdbuf+1,NULL,16);
        if(cmdbufindex>=5){
          
          param=strtol(cmdbuf+4,NULL,16);
        }

        //sprintf(debug,"Debug %02X,%04X",cmd,param);
        //SerialUSB.println(debug);
        if(cmd==0xF2){
          SSV3232_SetEmissivity(param);
        }
      }
      cmdbufindex=0;
      return;
    }
    if(cmdbufindex<30){
      cmdbuf[cmdbufindex++]=c;
    }
  }
}




// the loop function runs over and over again forever
void loop() {
    //byte status = SSV3232_GetStatus();
    //SerialUSB.println(status,HEX);

    while(!timerFlag){
      delay(1);
    }
    timerFlag=0;


    int atemp = SSV3232_GetAmbientTemp();
    SerialUSB.print("#03,");
    SerialUSB.println(atemp,HEX);

    int emiss = SSV3232_GetEmissivity();
    SerialUSB.print("#F2,");
    SerialUSB.println(emiss,HEX);


    SSV3232_GetArrayData();
    SerialUSB.println("#02");
    dump();

    proccmd();

    //delay(10);
}
