﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.Runtime.InteropServices;
using ComUtil;


namespace SSV3232_MON
{
    public partial class Form1 : Form
    {
        private int vtoRight;
        private int vtoBottom;
        private int pboxframe;

        private int scaleBoxWidth;
        private int pictureBoxGap;
        
        private Cmd cmd;
        Bitmap thermalImage;
        Bitmap scaleImage;




        private int min_temperature;
        private int max_temperature;

        private int[,] temperature_array;
        private bool upsideDown;
        private bool mirrorLR;





        private Int32[] colorTable;
        private int colorTableSize;

        private int drawCellSize;
        private int measX, measY;

        private bool setTempFirst;
        private double tempVal;

        public Form1()
        {
            InitializeComponent();

            Size sz, szcl;
            sz = this.Size;
            this.MinimumSize = sz;

            min_temperature = 0;
            max_temperature = 500;

            temperature_array = new int[32,32];
            upsideDown = true;
            mirrorLR = false;

            textBoxTempMin.Text = Convert.ToString((double)min_temperature / 10);
            textBoxTempMax.Text = Convert.ToString((double)max_temperature / 10);

            SetupColorTable();

            sz=pictureBoxThermalImage.Size;
            szcl = pictureBoxThermalImage.ClientSize;
            pboxframe = sz.Width - szcl.Width;
            int n=(szcl.Height<szcl.Width?szcl.Height:szcl.Width);
            n = (n / 32) * 32;
            pictureBoxThermalImage.ClientSize = new Size(n, n);

            scaleBoxWidth = pictureBox2.Width;
            pictureBoxGap = pictureBox2.Left - pictureBoxThermalImage.Right;
            
            vtoRight = this.Width-pictureBoxThermalImage.Right;
            vtoBottom = this.Height-pictureBoxThermalImage.Bottom;
            measX= measY=16;

            SetupItemPosition();

            DrawScale();

            cmd =new Cmd();
            DrawThermalImage();
            setTempFirst = true;

            string s = Properties.Settings.Default.portname;
            if(s.Length > 0)
            {
                serialPort1.PortName = s;
            }

            try
            {
                //serialPort1.Open();
            }
            catch
            {

            }
            timer1.Start();
        }

        //カラーテーブルの設定
        //黒→青→緑→赤→白
        private void SetupColorTable()
        {
            const int BLEN = 256;
            colorTableSize = BLEN * 4;
            colorTable = new Int32[colorTableSize];
            uint b, g, r;
            b = g = r = 0;
            int index = 0;
            for(uint i = 0; i < BLEN; i++)
            {
                b = i;
                colorTable[index++] = (int)(0xFF000000 + ((r << 16) + (g << 8) + b));
            }
            for (uint i = 0; i < BLEN; i++)
            {
                b = 255 - i;
                g = i;
                colorTable[index++] = (int)(0xFF000000 + ((r << 16) + (g << 8) + b));
            }
            for (uint i = 0; i < BLEN; i++)
            {
                g = 255 - i;
                r = i;
                colorTable[index++] = (int)(0xFF000000 + ((r << 16) + (g << 8) + b));
            }
            for (uint i = 0; i < BLEN; i++)
            {
                b= g = i;
                colorTable[index++] = (int)(0xFF000000 + ((r << 16) + (g << 8) + b));
            }
        }

        //指定したセルの温度を温度表示テキストボックスにセットする。
        //setTempFirst が trueのときは、引数をそのままセットするが、それ以降は1次フィルタ処理値を入れる
        void SetTemp(int temp)
        {
            if (temp < -10) return;
            double dtemp = (double)temp / 10;
            if (setTempFirst)
            {
                setTempFirst = false;
                tempVal = dtemp;
            }
            else
            {
                tempVal = tempVal * 0.9 + dtemp * 0.1;
            }
            textBoxTemp.Text = tempVal.ToString("F1");
        }







        void fillCell(Graphics g, Int32 colorval, int x, int y, int cellsize)
        {
            int l = x * cellsize;
            int t = y * cellsize;

           //colorval = (Int32)(0x7F00FF00);

            SolidBrush br = new SolidBrush(Color.FromArgb(colorval));
            g.FillRectangle(br, l, t, cellsize, cellsize);

            //g.FillRectangle(new SolidBrush(Color.LightGray), l, t, cellsize-1, cellsize);

            br.Dispose();


            if (x == measX && y == measY)
            {
                int frameColor = colorval ^ 0x00FFFFFF;
                Pen pen = new Pen(Color.FromArgb(frameColor));
                g.DrawRectangle(pen, l, t, cellsize-1, cellsize-1);
                pen.Dispose();
            }


        }





        //スケールのステップの候補。1℃、2.5℃、5℃…
        private readonly int[] scalePitchTable = { 10, 25, 50, 100, 250, 500, 1000 };   //温度の10倍の値

        //スケールの描画
        void DrawScale()
        {
            const int tbmargin = 10;
            const int colorScaleWindth = 16;
            Pen pen;

            Size sz = pictureBox2.ClientSize;

            if (sz.Width <= 0 || sz.Height <= 0) return;


            Bitmap bmp = new Bitmap(sz.Width, sz.Height);
            pictureBox2.Image = bmp;


            int h = sz.Height - tbmargin * 2;
            int w = colorScaleWindth;


            try
            {
                Graphics g = Graphics.FromImage(pictureBox2.Image);

                int ypos = tbmargin;
                for(int y = 0; y < h; y++)
                {
                    int colindex = colorTableSize * (h - y - 1) / h;
                    int color = colorTable[colindex];
                    pen = new Pen(Color.FromArgb(color));
                    g.DrawLine(pen, 0, ypos, w, ypos);
                    pen.Dispose();
                    ypos++;
                }



 

                int range = max_temperature - min_temperature;
                int i;
                for (i = 0; i < scalePitchTable.Length; i++)
                {
                    int n=range/scalePitchTable[i];
                    if (n <= 6) break; 
                }
                int scalePitch = scalePitchTable[i];
                int p;
                if (max_temperature > 0) p = max_temperature;
                else p = max_temperature - (scalePitch - 1);
                p=p/scalePitch * scalePitch;

                int l = colorScaleWindth+1;
                int r = l+10;

                pen = new Pen(Color.Black);
                Font font = new Font("MS UI Gothic", 8);

                while (p >= min_temperature)
                {
                    int y=(h-1)*(max_temperature-p)/ (max_temperature - min_temperature)+tbmargin;
                    g.DrawLine(pen, new Point(l, y), new Point(r, y));
                    g.DrawString(((double)p/10).ToString(), font, Brushes.Black, new Point(r+3, y-4));
                    p -= scalePitch;
                }
                font.Dispose();
                pen.Dispose();
                g.Dispose();

            }
            catch
            {

            }

        }

        

        //サーマルイメージの描画
        void DrawThermalImage()
        {
            //ピクチャーボックスのクライアントサイズは正方形で32の倍数になっていることを前提とする
            Size sz = pictureBoxThermalImage.ClientSize;
            int drawsize = sz.Width;
            drawCellSize = drawsize / 32;

            pictureBoxThermalImage.Image = thermalImage;
            Graphics g = Graphics.FromImage(pictureBoxThermalImage.Image);

            for (int y= 0; y < 32; y++)
            {
                for (int x = 0; x < 32; x++)
                {
                    int tempdat = temperature_array[x, y];  //cmd.databuf[index++];

                    int colorindex = (int)(colorTableSize * (double)(tempdat - min_temperature) / (max_temperature - min_temperature));
                    if (colorindex < 0) colorindex = 0;
                    else if (colorindex >= colorTableSize) colorindex = colorTableSize - 1;


                    fillCell(g, colorTable[colorindex], x, y, drawCellSize);
                }
            }
            g.Dispose();


        }

        //コマンドCmdクラスのデータバッファから、温度データ2次元配列へコピー
        private void copyToTemperatueArray()
        {
            int y = (upsideDown ? 31 : 0);
            int yd =(upsideDown ? -1 : 1);
            int x0 = (mirrorLR ? 31 : 0);
            int xd = (mirrorLR ? -1 : 1);

            int index = 0;

            for(int yc=0; yc < 32; yc++)
            {
                int x = x0;
                for (int xc=0; xc < 32; xc++)
                {
                    temperature_array[x, y] = cmd.databuf[index++];
                    x += xd;
                }
                y += yd;
            }
        }

        //タイマー処理。データ受信はここで処理している。
        private void timer1_Tick(object sender, EventArgs e)
        {
            if(this.WindowState == FormWindowState.Minimized) return;

            timer1.Enabled = false;
            try
            {
                if (serialPort1.IsOpen)
                {
                    int len = serialPort1.BytesToRead;
                    if (len > 5120)
                    {
                        serialPort1.DiscardInBuffer();
                        cmd.Init();
                        System.Diagnostics.Debug.WriteLine("***** BufferOverflow *****");
                    }
                    else{
                        Byte[] buf = new Byte[len]; 
                        serialPort1.Read(buf, 0, len);
                        for (int i = 0; i < len; i++)
                        {
                            //byte c = (byte)serialPort1.ReadByte();
                            byte c=buf[i];
                            Cmd.CMDTYPE ct = cmd.SetChar(c);
                            switch (ct)
                            {
                                case Cmd.CMDTYPE.CT_EMISS:
                                    System.Diagnostics.Debug.WriteLine("Emissivity: " + cmd.emiss);
                                    break;
                                case Cmd.CMDTYPE.CT_TAMB:
                                    System.Diagnostics.Debug.WriteLine("Ambient temperature: " + cmd.tamb);
                                    break;
                                case Cmd.CMDTYPE.CT_DATA:
                                    copyToTemperatueArray();
                                    DrawThermalImage();
                                    SetTemp(temperature_array[measX, measY]);
                                    break;
                            }
                        }
                    }
                }
            }
            catch
            {

            }
            timer1.Enabled = true;

        }

        //描画サイズ変更にともなう温度スケールの位置合わせ
        private void moveScaleBox()
        {
            int left = pictureBoxThermalImage.Right + pictureBoxGap;
            int top = pictureBoxThermalImage.Top;
            int height = pictureBoxThermalImage.Height;
            
            pictureBox2.Size = new Size(scaleBoxWidth,height);
            pictureBox2.Location = new Point(left,top);

            DrawScale();
        }

        private void SetupItemPosition()
        {
            int w = this.Width - vtoRight - pictureBoxThermalImage.Left - pboxframe;
            int h = this.Height - vtoBottom - pictureBoxThermalImage.Top - pboxframe;
            int n = (w < h ? w : h);
            n = (n / 32) * 32;
            pictureBoxThermalImage.ClientSize = new Size(n, n);

            if (this.WindowState == FormWindowState.Minimized) return;

            thermalImage = new Bitmap(n, n);
            moveScaleBox();
            DrawThermalImage();
        }



        //ウィンドウサイズ変更時の処理
        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            SetupItemPosition();
        }

        //温度レンジ設定ボタンクリック時の処理
        private void buttonSetTempRange_Click(object sender, EventArgs e)
        {
            string mintext = textBoxTempMin.Text;
            string maxtext = textBoxTempMax.Text;
            double dmin, dmax;
            try { dmin = Convert.ToDouble(mintext); } catch { dmin = -10.0; }
            try { dmax = Convert.ToDouble(maxtext); } catch { dmax = 250.0; }
            if (dmin < -10.0) dmin = -10.0;
            if (dmax > 250.0) dmax = 250.0;
            if (dmin > dmax) { double d = dmax; dmax = dmin; dmin = d; }
            min_temperature = (int)(dmin * 10);
            max_temperature = (int)(dmax * 10);

            textBoxTempMin.Text = Convert.ToString((double)min_temperature / 10);
            textBoxTempMax.Text = Convert.ToString((double)max_temperature / 10);
            DrawScale();
            DrawThermalImage();
        }

        //放射率設定ボタンのクリック時の処理
        private void buttonSetEmiss_Click(object sender, EventArgs e)
        {
            if (!serialPort1.IsOpen) return;
            string emissText = textBoxEmiss.Text;
            double d;
            try { d = Convert.ToDouble(emissText); } catch { d = 1.0; }
            if (d < 0.01) d = 0.01;
            else if (d > 1.0) d = 1.0;
            textBoxEmiss.Text = Convert.ToString(d);
            int setval = (int)(d * 1000);
            string s = string.Format("WF2,{0,0:X4}\r", setval);
            serialPort1.Write(s);
        }

        //COMポート設定ボタンのクリック時の処理
        private void buttonSetCom_Click(object sender, EventArgs e)
        {
            SetComForm scf = new SetComForm();
            scf.PortName = serialPort1.PortName;
            DialogResult dr = scf.ShowDialog();
            if (dr == DialogResult.OK || dr == DialogResult.Cancel)
            {
                try
                {
                    serialPort1.Close();
                }
                catch
                {

                }
            }

            if (dr == DialogResult.OK)
            {
                string s = scf.PortName;
                if (s.Length > 0)
                {
                    serialPort1.PortName = s;
                    try
                    {
                        serialPort1.Open();
                    }
                    catch
                    {
                    }
                }
                if (!serialPort1.IsOpen)
                {
                    MessageBox.Show("COMポートを開けません", "SSV32x32");
                    return;
                }

                Properties.Settings.Default.portname = s;
                Properties.Settings.Default.Save();
            }
        }

        //サーマルイメージをクリックしたときの処理。温度測定位置の設定。
        private void pictureBoxThermalImage_MouseClick(object sender, MouseEventArgs e)
        {
            Point pt = e.Location;
            measX = pt.X / drawCellSize;
            measY = pt.Y / drawCellSize;
            setTempFirst = true;
        }
    }

}
