﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SSV3232_MON
{
    public class Cmd
    {
        StringBuilder sb;

        private enum CMDSTATE
        {
            CS_IDLE,
            CS_CMD,
            CS_TEXT_PARAM
        };

        public enum CMDTYPE
        {
            CT_N,
            CT_TAMB,
            CT_EMISS,
            CT_DATA
        };

        private const int DATABUF_SIZE = 1024;
        private const int MAX_PARAMS = 8;

        public double tamb;
        public double emiss;
        public int[] databuf;

        private CMDTYPE command;

        //private string[] strParam;
        private int paramIndex;
        //public int paramSize;

        private CMDSTATE cmdstate;

        public Cmd()
        {
            cmdstate = CMDSTATE.CS_IDLE;
            sb = new StringBuilder();
            databuf = new int[DATABUF_SIZE];
            for(int i = 0; i < databuf.Length; i++) databuf[i] = -20;
            //strParam = new string[MAX_PARAMS]; 
        }

        public void Init()
        {
            cmdstate = CMDSTATE.CS_IDLE;
        }

        public CMDTYPE SetChar(byte c)
        {
            if (cmdstate == CMDSTATE.CS_IDLE)
            {
                if (c == (byte)('#'))
                {
                    cmdstate = CMDSTATE.CS_CMD;
                }
            }
            else
            {
                if (c == ',' || c == '\r')
                {
                    string wd = sb.ToString();
                    sb.Length = 0;

                    uint d;
                    try
                    {
                        d = Convert.ToUInt32(wd, 16);
                    }
                    catch
                    {
                        d = 0;
                    }
                    

                    if (cmdstate == CMDSTATE.CS_CMD)
                    {
                        switch (d)
                        {
                            case 0xF2:
                                command = CMDTYPE.CT_EMISS;
                                cmdstate = CMDSTATE.CS_TEXT_PARAM;
                                break;
                            case 3:
                                command = CMDTYPE.CT_TAMB;
                                cmdstate = CMDSTATE.CS_TEXT_PARAM;
                                break;
                            case 2:
                                command = CMDTYPE.CT_DATA;
                                paramIndex = 0;
                                cmdstate = CMDSTATE.CS_TEXT_PARAM;
                                break;
                            default:
                                cmdstate = CMDSTATE.CS_IDLE;
                                return CMDTYPE.CT_N;
                        }
                    }
                    else
                    {
                        switch (command)
                        {
                            case CMDTYPE.CT_EMISS:
                                emiss = (double)d / 1000;
                                cmdstate = CMDSTATE.CS_IDLE;
                                return CMDTYPE.CT_EMISS;

                            case CMDTYPE.CT_TAMB:
                                tamb = (double)d / 10;
                                cmdstate = CMDSTATE.CS_IDLE;
                                return CMDTYPE.CT_TAMB;

                            case CMDTYPE.CT_DATA:
                                databuf[paramIndex++] = (int)d;
                                if (paramIndex >= DATABUF_SIZE)
                                {
                                    cmdstate = CMDSTATE.CS_IDLE;
                                    return CMDTYPE.CT_DATA;
                                }
                                break;
                            default :
                                cmdstate = CMDSTATE.CS_IDLE;
                                break;
                        }
                    }
                }
                else
                {
                    if (c!='\n') {
                        sb.Append((char)c);
                    }
                }
            }
            return CMDTYPE.CT_N;

        }
    }   //end class cmd
}
